@@ -1,6 +1,8 @@
 weights.matrix <- function(match.matrix, treat) {
 
-  weights <- weights_matrix(match.matrix, treat)
+  if (!is.integer(match.matrix)) match.matrix <- charmm2nummm(match.matrix, treat)
+
+  weights <- weights_matrixC(match.matrix, treat)
 
   if (sum(weights)==0)
     stop("No units were matched.", call. = FALSE)
